#include <iostream>
#include "ncfile.h"
#include <cstdio>

using namespace ncfile;

// file: ccm2iop.C
//     - converts CCM output to SCM input (IOP)



int main( int argc, char* argv[] ) {

      // 
      // note the use of NcDoubleVar declarations - this
      // prevents loss of accuracy that would occur if
      //  variables were declared as NcFloatVar
      //
    try {
        NcFile input( argv[1], NcFile::WRITE );

        input.enterDefineMode();
        
          // add the global attribute that marks this file as a ccmiop file
        input.addAttribute( NcCharAtt( string( "CCM_GENERATED_FORCING" ), "generated by CCM3.6, converted by ccm2iop" ) ); 

          // fix the names of the variables
         
        NcDoubleVar var = input.variable( "PsB4_PHY" );
        var.rename( "Ps" );

        var = input.variable( "UB4_PHY" );
        var.rename( "u" );

        var = input.variable( "VB4_PHY" );
        var.rename( "v" );

        var = input.variable( "TsB4_PHY" );
        var.rename( "Tg" );

        var = input.variable( "T" );
        var.rename( "T_ORIG" );

        var = input.variable( "TB4_PHY" );
        var.rename( "T" );

        var = input.variable( "QB4_PHY" );
        var.rename( "q" );

        var = input.variable( "divt3d" );
        var.rename( "divT3d" );

        var = input.variable( "divq3d" );

        var = input.variable( "OMEGA" );
        var.rename( "omega" );

        var = input.variable( "PHIS" );
        var.rename( "phis" );

        var = input.variable( "PRECT" );
        var.rename( "Prec" );

        var = input.variable( "LHFLX" );
        var.rename( "lhflx" );

        var = input.variable( "SHFLX" );
        var.rename( "shflx" );

        var = input.variable( "TREFHT" );
        var.rename( "Tsair" );

        NcIntVar ivar;
        ivar = input.variable( "nbdate" );
        ivar.rename( "bdate" );

        ivar = input.variable( "datesec" );
        ivar.rename( "tsec" );

	NcDimension dims[4];
        NcDimension timeDim = input.dimension( "time" );
        NcDimension latDim = input.dimension( "lat" );
        NcDimension levDim = input.dimension( "lev" );
        NcDimension lonDim = input.dimension( "lon" );
        dims[0] = timeDim;
        dims[1] = latDim;
        dims[2] = lonDim;

        NcDoubleVar fixmas = NcDoubleVar( "fixmas", dims, 3 );
        fixmas.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        fixmas.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( fixmas );

        NcDoubleVar beta = NcDoubleVar( "beta", dims, 3 );
        beta.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        beta.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( beta );

        NcDoubleVar afixq = NcDoubleVar( "afixq", dims, 3 );
        afixq.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        afixq.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( afixq );

        NcDoubleVar afixcldl = NcDoubleVar( "afixcldl", dims, 3 );
        afixcldl.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        afixcldl.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( afixcldl );

        NcDoubleVar afixcldi = NcDoubleVar( "afixcldi", dims, 3 );
        afixcldi.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        afixcldi.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( afixcldi );

        dims[0] = timeDim;
        dims[1] = levDim;
        dims[2] = latDim;
        dims[3] = lonDim;
        NcDoubleVar dqfxq = NcDoubleVar( "dqfxq", dims, 4 );
        dqfxq.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        dqfxq.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( dqfxq );

        NcDoubleVar dqfxcldl = NcDoubleVar( "dqfxcldl", dims, 4 );
        dqfxcldl.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        dqfxcldl.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( dqfxcldl );

        NcDoubleVar dqfxcldi = NcDoubleVar( "dqfxcldi", dims, 4 );
        dqfxcldi.addAttribute( NcCharAtt( "long_name", "mass fixer" ) );
        dqfxcldi.addAttribute( NcCharAtt( "units", "multiplier" ) );
        input.addVariable( dqfxcldi );

        input.endDefineMode();

          // fix the tsec variable 
          //
          // in ccm output, the tsec variable wraps
          // at the end of 24 hours, but in sccm iop
          // datasets, tsec is monotonically increasing
        
        NcIntVar tsec = input.variable( "tsec" ); 
        tsec.read();
        int steplen = tsec[1] - tsec[0];
        for ( size_t i=0; i<tsec.size(); i++ ) 
            tsec[i] = steplen * i;
        tsec.write();
        
          // fix lon variable 0:360 -> -180:180
          // sccm expects the longitude in range of -180:180
        NcDoubleVar lon = input.variable( "lon" );
        lon.read();
        for (size_t i=0;i<lon.size(); i++ )
            if ( lon[i] > 180.0 )
                lon[i] -= 360.0;
        lon.write();
        
          // fix the lev variable mb -> pa
          // sccm expects the pressure data in pascals
        NcDoubleVar lev = input.variable("lev");
        lev.read();
        for ( size_t i=0; i<lev.size(); i++ ) 
            lev[i] *= 100.0;
        lev.write();
        
          //
          //
          // fix the values for ps, u and v: put ps_p into
          // ps at nstep >=2 using the value of ps_p from previous step
          // at nstep < 2, shift the values for ts, ps, u and v over by one step
          // 
          NcDimension timedim = input.dimension( "time" );
          int ntime = timedim.size();
        

	  NcDoubleVar ps = input.variable("Ps");
	  NcDoubleVar ps_ad = input.variable("PS_AD");
	  for ( int i=1; i<ntime;i++ ) {
	    ps_ad.readRecord( i-1 );
	    for ( size_t j = 0; j<ps.size()/ntime; j++ )
	    ps[j] = ps_ad[j];
	    ps.writeRecord( i );
	  }

	  NcDoubleVar FIXMAS = input.variable("FIXMAS");
	  fixmas = input.variable("fixmas");
	  fixmas[0]=1.0;
          fixmas.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    FIXMAS.readRecord( i-1 );
	    for ( size_t j = 0; j<FIXMAS.size()/ntime; j++ )
	      fixmas[j] = FIXMAS[j];
	    fixmas.writeRecord( i );
	  }
        
	  NcDoubleVar BETA = input.variable("BETA");
	  beta = input.variable("beta");
	  beta[0]=0.0;
          beta.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    BETA.readRecord( i-1 );
	    for ( size_t j = 0; j<BETA.size()/ntime; j++ )
	      beta[j] = BETA[j];
	    beta.writeRecord( i );
	  }

	  NcDoubleVar AFIXQ = input.variable("AFIXQ");
	  afixq = input.variable("afixq");
	  afixq[0]=0.0;
          afixq.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    AFIXQ.readRecord( i-1 );
	    for ( size_t j = 0; j<AFIXQ.size()/ntime; j++ )
	      afixq[j] = AFIXQ[j];
	    afixq.writeRecord( i );
	  }
        
	  NcDoubleVar AFIXCLDL = input.variable("AFIXCLDL");
	  afixcldl = input.variable("afixcldl");
	  afixcldl[0]=0.0;
          afixcldl.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    AFIXCLDL.readRecord( i-1 );
	    for ( size_t j = 0; j<AFIXCLDL.size()/ntime; j++ )
	      afixcldl[j] = AFIXCLDL[j];
	    afixcldl.writeRecord( i );
	  }
        
	  NcDoubleVar AFIXCLDI = input.variable("AFIXCLDI");
	  afixcldi = input.variable("afixcldi");
	  afixcldi[0]=0.0;
          afixcldi.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    AFIXCLDI.readRecord( i-1 );
	    for ( size_t j = 0; j<AFIXCLDI.size()/ntime; j++ )
	      afixcldi[j] = AFIXCLDI[j];
	    afixcldi.writeRecord( i );
	  }
        
	  NcDoubleVar DQFXQ = input.variable("DQFXQ");
	  dqfxq = input.variable("dqfxq");
	  dqfxq[0]=0.0;
          dqfxq.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    DQFXQ.readRecord( i-1 );
	    for ( size_t j = 0; j<DQFXQ.size()/ntime; j++ )
	      dqfxq[j] = DQFXQ[j];
	    dqfxq.writeRecord( i );
	  }
        
	  NcDoubleVar DQFXCLDL = input.variable("DQFXCLDL");
	  dqfxcldl = input.variable("dqfxcldl");
	  dqfxcldl[0]=0.0;
          dqfxcldl.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    DQFXCLDL.readRecord( i-1 );
	    for ( size_t j = 0; j<DQFXCLDL.size()/ntime; j++ )
	      dqfxcldl[j] = DQFXCLDL[j];
	    dqfxcldl.writeRecord( i );
	  }
        
	  NcDoubleVar DQFXCLDI = input.variable("DQFXCLDI");
	  dqfxcldi = input.variable("dqfxcldi");
	  dqfxcldi[0]=0.0;
          dqfxcldi.writeRecord( 0 );
	  for ( int i=1; i<ntime;i++ ) {
	    DQFXCLDI.readRecord( i-1 );
	    for ( size_t j = 0; j<DQFXCLDI.size()/ntime; j++ )
	      dqfxcldi[j] = DQFXCLDI[j];
	    dqfxcldi.writeRecord( i );
	  }
        

          NcDoubleVar u = input.variable("u");
          NcDoubleVar u_p = input.variable("U_AD");
          for ( int i=1; i<ntime;i++ ) { 
	    u_p.readRecord( i-1 );
	    for ( size_t j = 0; j<u.size()/ntime; j++ )
	      u[j] = u_p[j];
	    u.writeRecord( i );
          }

          NcDoubleVar v = input.variable("v");
          NcDoubleVar v_p = input.variable("V_AD");
          for ( int i=1; i<ntime;i++ ) {
            v_p.readRecord( i-1 );
            for ( size_t j = 0; j<v.size()/ntime; j++ )
	      v[j] = v_p[j];
	    v.writeRecord( i );
	  }

          NcDoubleVar T = input.variable("T");
	  NcDoubleVar T_p = input.variable("T_AD");
          for ( int i=1; i<ntime;i++ ) {
            T_p.readRecord( i-1 );
	    for ( size_t j = 0; j<T.size()/ntime; j++ )
	      T[j] = T_p[j];
	    T.writeRecord( i );
	  }

          NcDoubleVar Q = input.variable("q");
          NcDoubleVar Q_p = input.variable("Q_AD");
          for ( int i=1; i<ntime;i++ ) {
            Q_p.readRecord( i-1 );
	    for ( size_t j = 0; j<Q.size()/ntime; j++ )
	      Q[j] = Q_p[j];
	    Q.writeRecord( i );
	  }

    } catch ( NcErr e ) {
        cerr << '\n' << argv[0] << ": ERROR: " << e.toString() << endl;
        exit( -1 );
    }
}



